import functools
import os
import sys

import click
import git

from ginteract import interact


@click.group()
def main():
    pass


def command(*args, **kwargs):
    def decorator(func):
        # noinspection PyTypeChecker
        @click.command(*args, **kwargs)
        @functools.wraps(func)
        def wrapper(*a, **kw):
            repo = git.Repo(os.getcwd(), search_parent_directories=True)
            branches = sorted(repo.branches, key=lambda branch: branch.commit.committed_date, reverse=True)
            branches = [str(branch) for branch in branches]
            current = str(repo.active_branch)
            branches = list(filter(lambda branch: branch != current, branches))
            cmd = func(repo, branches, current, *a, **kw)
            try:
                cmd()
            except git.exc.GitCommandError as e:
                click.echo(e)

        main.add_command(wrapper)
        return wrapper

    return decorator


@command('checkout')
def checkout(repo, branches, _):
    choice = interact.prompt(branches)
    return lambda: repo.git.checkout(choice)


@command('merge')
def merge(repo, branches, current):
    choice = interact.prompt(branches, '{} <- '.format(current))
    return lambda: repo.git.merge(choice)


@command('delete')
def delete(repo, branches, _):
    choices = interact.prompt(branches, multiple=True)
    if choices and not click.confirm('Delete {}?'.format(', '.join(choices))):
        raise click.Abort

    def delete_branches():
        for branch in choices:
            repo.git.branch('-d', branch)

    return delete_branches


if __name__ == '__main__':
    sys.exit(main())



import os
import tempfile
import traceback
import unittest
import uuid

import click.testing
import git

from ginteract import main


def click_runner(func):
    def wrapper(instance, *args, **kwargs):
        runner = click.testing.CliRunner()
        result = func(instance, runner, *args, **kwargs)

        if result.exit_code != 0:
            etype, value, tb = result.exc_info
            print(''.join(traceback.format_exception(etype, value, tb)))
            raise Exception('Exit code is {}'.format(result.exit_code))

    return wrapper


class TestMain(unittest.TestCase):
    def setUp(self):
        self.tmpdir = tempfile.TemporaryDirectory()
        os.chdir(self.tmpdir.name)

        self.repo = git.Repo.init(self.tmpdir.name)
        self.make_commit()

        self.repo.create_head('branch0')
        self.make_commit()

        self.repo.create_head('branch1')
        self.make_commit()

    def make_commit(self):
        filename = str(uuid.uuid4())
        path = os.path.join(self.tmpdir.name, filename)
        with open(path, 'w') as fp:
            fp.write(filename)  # write filename to file
        self.repo.index.add([path])
        self.repo.git.commit('-m', filename)

    @click_runner
    def test_checkout(self, runner):
        result = runner.invoke(main.checkout, input='branch0')
        assert str(self.repo.active_branch) == 'branch0'
        return result

    @click_runner
    def test_merge(self, runner):
        self.repo.git.checkout('master')
        result = runner.invoke(main.merge, input='branch1')
        assert 3 == len(list(self.repo.iter_commits('master')))
        return result

    @click_runner
    def test_delete(self, runner):
        result = runner.invoke(main.delete, input='branch0 branch1\ny')
        assert 1 == len(self.repo.branches)
        return result

    def tearDown(self):
        self.tmpdir.cleanup()



from setuptools import setup

with open('README.md') as fp:
    readme = fp.read()

setup(
    name='ginteract',
    packages=['ginteract'],
    version='0.2.2',
    description='An interactive Git client!',
    long_description=readme,
    author='Jacob Smith',
    author_email='jacob.smith@unb.ca',
    url='http://github.com/jacsmith21/ginteract',
    license='MIT',
    install_requires=[
        'GitPython',
        'inquirer',
        'click'
    ],
    entry_points={
        'console_scripts': ['ginteract=ginteract.main:main']
    }
)




import io
import termios

import click
import inquirer


class Choices(click.Choice):
    """The choice type allows a value to be checked against a fixed set of
    supported values.  All of these values have to be strings.

    See :ref:`choice-opts` for an example.
    """
    name = 'choices'

    def convert(self, value: str, param, ctx):
        for choice in value.split():
            choice = choice.strip()
            if choice not in self.choices:
                self.fail('invalid choice: %s. (choose from %s)' % (value, ', '.join(self.choices)), param, ctx)
        else:
            return value

    def __repr__(self):
        return 'Choices(%r)' % list(self.choices)


def prompt(choices, message='Choice', multiple=False):
    cls = inquirer.Checkbox if multiple else inquirer.List
    question = [cls('key', message=message, choices=choices)]
    try:
        choice = inquirer.prompt(question)
        if choice is None:
            raise click.Abort
        choice = choice['key']
    except (termios.error, io.UnsupportedOperation):
        text = '{} ({})'.format(message, ', '.join(choices))
        if multiple:
            choice = click.prompt(text, default=choices[0], type=Choices(choices))
            choice = choice.split()
        else:
            choice = click.prompt(text, default=choices[0], type=click.Choice(choices))

    return choice
